@@ -1,44 +1,28 @@
+# $Id: /mirror/coderepos/lang/perl/Data-Localize/trunk/lib/Data/Localize/Localizer.pm 100948 2009-02-20T04:07:52.965534Z daisuke  $
 
 package Data::Localize::Localizer;
 use utf8;
-use Any::Moose;
-use Carp ();
+use Moose::Role;
+use Moose::Util::TypeConstraints;
 
-has _localizer => (
+requires 'register';
+
+has 'style' => (
     is => 'rw',
-    isa => 'Data::Localize',
-    weak_ref => 1,
+    isa => enum([qw(gettext maketext)]),
+    default => 'maketext',
 );
 
-has formatter => (
-    is => 'ro',
-    isa => 'Data::Localize::Format',
-    required => 1,
-    lazy_build => 1,
-    handles => { format_string => 'format' },
-);
-
-no Any::Moose;
-
-sub _build_formatter {
-    Any::Moose::load_class('Data::Localize::Format::Maketext');
-    return Data::Localize::Format::Maketext->new();
-}
-
-sub register {
-    my ($self, $loc) = @_;
-    if ($self->_localizer) {
-        Carp::confess("Localizer $self is already attached to another Data::Localize object ($loc)");
-    }
-    $self->_localizer( $loc );
-}
+no Moose::Role;
+no Moose::Util::TypeConstraints;
 
 sub localize_for {
     my ($self, %args) = @_;
     my ($lang, $id, $args) = @args{ qw(lang id args) };
 
-    my $value = $self->get_lexicon($lang, $id) or return ();
-    if (Data::Localize::DEBUG()) {
+    my $lexicon = $self->lexicon_get($lang) or return ();
+    my $value = $lexicon->{ $id };
+    if (&Data::Localize::DEBUG) {
         print STDERR "[Data::Localize::Localizer]: localize_for - $id -> ",
             defined($value) ? $value : '(null)', "\n";
     }
@@ -46,7 +30,20 @@ sub localize_for {
     return ();
 }
 
-__PACKAGE__->meta->make_immutable();
+sub format_string {
+    my ($self, $value, @args) = @_;
+
+    my $style = $self->style;
+print STDERR "$self -> localizing '$value' with (@args), style is $style\n" if Data::Localize::DEBUG();
+    if ($style eq 'gettext') {
+        $value =~ s/%(\d+)/ $args[$1 - 1] || '' /ge;
+    } elsif ($style eq 'maketext') {
+        $value =~ s/\[_(\d+)\]/$args[$1 - 1] || ''/ge;
+    } else {
+        confess "Unknown style: $style";
+    }
+    return $value;
+}
 
 1;
 
@@ -54,24 +51,19 @@ __END__
 
 =head1 NAME
 
-Data::Localize::Localizer - Localizer Base Class
+Data::Localize::Localizer - Localizer Role
 
 =head1 SYNOPSIS
 
     package MyLocalizer;
     use Moose;
 
-    extends 'Data::Localize::Localizer';
+    with 'Data::Localize::Localizer';
 
     no Moose;
 
 =head1 METHODS
 
-=head2 register
-
-Does basic registration for the localizer. If you're overriding
-this method, be sure to call the super class' register() method!
-
 =head2 localize_for
 
 =head2 format_string
